/*
 ************************************************************************
 *
 * This is a glue code that "renames" HTTPFS's httpe_open() into open(),
 * so that our httpe_open() takes place of (interposes upon) 
 * the ordinary open(2) system call.
 * The same for httpe_close(), httpe_stat(), httpe_lstat(), and fopen()
 *
 * $Id: httpfs_glue.c,v 1.2 1999/05/06 00:22:36 oleg Exp oleg $
 *
 ************************************************************************
 */ 

#include <sys/types.h>
#include <sys/stat.h>
#if defined(linux)
#include <fcntlbits.h>
#else
#include <sys/file.h>
#include <sys/fcntl.h>
#endif
#include <errno.h>
#include <stdio.h>

extern int httpe_open(const char *filename, const int mode, const int mask);
extern int httpe_close(const int fildes);
int _close(const int fildes);
extern int httpe_stat(const char *path, struct stat *buf);
extern int httpe_lstat(const char *path, struct stat *buf);

int open(const char *filename, const int mode, const int mask)
{
#if !defined(DEBUG)
  return httpe_open(filename,mode,mask);
#else
  int result;
  printf("\nOpening '%s'...",filename);
  result = httpe_open(filename,mode,mask);
  printf("\nresult... %d",result);
  if(result < 0 )
   perror("opening error");
  return result;
#endif
}

int close(const int fildes)
{
#if !defined(DEBUG)
  return httpe_close(fildes);
#else
  int result;
  printf("\nClosing '%d'...",fildes);
  result = httpe_close(fildes);
  printf("\nresult... %d",result);
  if(result < 0 )
   perror("closing error");
  return result;
#endif
}

#if defined(linux)
int __close(const int fildes)
#else
int _close(const int fildes)
#endif
{
#if !defined(DEBUG)
  return httpe_close(fildes);
#else
  int result;
  printf("\nClosing '%d'...",fildes);
  result = httpe_close(fildes);
  printf("\nresult... %d",result);
  if(result < 0 )
   perror("closing error");
  return result;
#endif
}


int stat(const char *path, struct stat *buf)
{
#if !defined(DEBUG)
  return httpe_stat(path, buf);
#else
  int result;
  printf("\nstat of '%s'...",path);
  result = httpe_stat(path, buf);
  printf("\nresult... %d",result);
  if(result < 0 )
   perror("stat error");
  return result;
#endif
}

int lstat(const char *path, struct stat *buf)
{
#if !defined(DEBUG)
  return httpe_lstat(path, buf);
#else
  int result;
  printf("\nlstat of '%s'...",path);
  result = httpe_lstat(path, buf);
  printf("\nresult... %d",result);
  if(result < 0 )
   perror("lstat error");
  return result;
#endif
}

FILE *fopen(const char *filename, const char *mode)
{
  static struct ModeString { const char * str; const int bin_value; }
  mode_strings [] =
  { {"r+", O_RDWR}, {"rb+", O_RDWR}, {"w+", O_RDWR|O_CREAT|O_TRUNC},
    {"wb+", O_RDWR|O_CREAT|O_TRUNC},
    {"a+", O_RDWR|O_CREAT|O_APPEND},
    {"ab+", O_RDWR|O_CREAT|O_APPEND},
    {"r", O_RDONLY}, {"w", O_WRONLY|O_CREAT|O_TRUNC},
    {"a", O_WRONLY|O_CREAT|O_APPEND}
  };
  register struct ModeString * p;
  int fd = -1;
  errno = EINVAL;
  for(p=mode_strings; p< mode_strings + sizeof(mode_strings)/sizeof(mode_strings[0]);
      p++)
    if( strncmp(mode,p->str,strlen(p->str)) == 0 )
      {
        fd = httpe_open(filename,p->bin_value,0777);
	break;
      }
  if( fd < 0 )
    return (FILE*)0;
  return fdopen(fd,mode);
}

